@@ -6,11 +6,11 @@ WWW::Mechanize - Handy web browsing in a Perl object
 
 =head1 VERSION
 
-Version 1.10
+Version 1.14
 
 =cut
 
-our $VERSION = "1.10";
+our $VERSION = "1.14";
 
 =head1 SYNOPSIS
 
@@ -75,6 +75,9 @@ you can also use any of L<LWP::UserAgent>'s methods.
 
     $mech->add_header($name => $value);
 
+Please note that Mech does NOT support JavaScript.  Please check the
+FAQ in WWW::Mechanize::FAQ for more.
+
 =head1 IMPORTANT LINKS
 
 =over 4
@@ -100,6 +103,7 @@ use LWP::UserAgent 2.003;
 use HTML::Form 1.00;
 use HTML::TokeParser;
 use URI::URL;
+use UNIVERSAL qw( isa );
 
 use base 'LWP::UserAgent';
 
@@ -198,7 +202,8 @@ sub new {
     while ( my($key,$value) = each %passed_parms ) {
         if ( exists $mech_parms{$key} ) {
             $mech_parms{$key} = $value;
-        } else {
+        }
+        else {
             $parent_parms{$key} = $value;
         }
     }
@@ -271,7 +276,8 @@ sub agent_alias {
 
     if ( defined $known_agents{$alias} ) {
         return $self->agent( $known_agents{$alias} );
-    } else {
+    }
+    else {
         $self->warn( qq{Unknown agent alias "$alias"} );
         return $self->agent();
     }
@@ -335,10 +341,9 @@ if there's no current request.
 sub reload {
     my $self = shift;
 
-    return unless $self->{req};
+    return unless my $req = $self->{req};
 
-    return unless defined(my $request = $self->{req});
-	$self->_update_page($request, $self->_make_request( $request, @_ ));
+    $self->_update_page( $req, $self->_make_request( $req, @_ ) );
 }
 
 =head2 $mech->back()
@@ -494,13 +499,16 @@ sub content {
                 $tree->eof();
                 $tree->elementify(); # just for safety
                 $content = $tree->as_text();
-            } else {
+            }
+            else {
                 $self->die( qq{Unknown format parameter "$arg"} );
             };
-        } elsif ($cmd eq 'base_href') {
+        }
+        elsif ($cmd eq 'base_href') {
             $arg ||= $self->base;
             $content=~s/<head>/<head>\n<base href="$arg">/;
-        } else {
+        }
+        else {
             $self->die( qq{Unknown named argument "$cmd"} );
         }
     }
@@ -689,7 +697,8 @@ sub find_link {
         if ( _match_any_link_parms($link,\%parms) ) {
             if ( $wantall ) {
                 push( @matches, $link );
-            } else {
+            }
+            else {
                 ++$nmatches;
                 return $link if $nmatches >= $parms{n};
             }
@@ -752,7 +761,8 @@ sub _clean_keys {
                 delete $parms->{$key};
                 next;
             }
-        } else {
+        }
+        else {
             if ( $val_regex ) {
                 $self->warn( qq{$val passed as '$key' is a regex} );
                 delete $parms->{$key};
@@ -883,7 +893,8 @@ sub find_image {
         if ( _match_any_image_parms($image,\%parms) ) {
             if ( $wantall ) {
                 push( @matches, $image );
-            } else {
+            }
+            else {
                 ++$nmatches;
                 return $image if $nmatches >= $parms{n};
             }
@@ -971,9 +982,10 @@ sub form_number {
     if ($self->{forms}->[$form-1]) {
         $self->{form} = $self->{forms}->[$form-1];
         return $self->{form};
-    } else {
+    }
+    else {
         $self->warn( "There is no form numbered $form" );
-        return undef;
+        return;
     }
 }
 
@@ -997,7 +1009,8 @@ sub form_name {
         $self->warn( "There are ", scalar @matches, " forms named $form.  The first one was used." )
             if @matches > 1;
         return $self->{form} = $matches[0];
-    } else {
+    }
+    else {
         $self->warn( qq{ There is no form named "$form"} );
         return undef;
     }
@@ -1024,10 +1037,12 @@ sub field {
     my $form = $self->{form};
     if ($number > 1) {
         $form->find_input($name, undef, $number)->value($value);
-    } else {
+    }
+    else {
         if ( ref($value) eq "ARRAY" ) {
             $form->param($name, $value);
-        } else {
+        }
+        else {
             $form->value($name => $value);
         }
     }
@@ -1082,7 +1097,8 @@ sub select {
             # (see INPUTS section of `perldoc HTML::Form`)
             if (@inputs == 1) {
                 @values = $inputs[0]->possible_values();
-            } else {
+            }
+            else {
                 foreach my $input (@inputs) {
                     my @possible = $input->possible_values();
                     push @values, pop @possible;
@@ -1099,13 +1115,16 @@ sub select {
                     }
                     push @$value, $values[$_ - 1];  # might be undef
                 }
-            } elsif (!ref($n) && $n =~ /^\d+$/) {
+            }
+            elsif (!ref($n) && $n =~ /^\d+$/) {
                 $value = $values[$n - 1];           # might be undef
-            } else {
+            }
+            else {
                 $self->warn('"n" value is not a positive integer or an array ref');
                 return;
             }
-        } else {
+        }
+        else {
             $self->warn('Hash value is invalid');
             return;
         }
@@ -1142,13 +1161,14 @@ sub set_fields {
     my $self = shift;
     my %fields = @_;
 
-    my $form = $self->current_form;
+    my $form = $self->current_form or $self->die( "No form defined" );
 
     while ( my ( $field, $value ) = each %fields ) {
         if ( ref $value eq 'ARRAY' ) {
             $form->find_input( $field, undef,
                          $value->[1])->value($value->[0] );
-        } else {
+        }
+        else {
             $form->value($field => $value);
         }
     } # while
@@ -1209,7 +1229,8 @@ sub set_visible {
                     last;
                 }
             } # while
-        } else {
+        }
+        else {
             while ( my $input = shift @inputs ) {
                 next if $input->type eq 'hidden';
                 $input->value( $value );
@@ -1292,7 +1313,8 @@ sub value {
     my $form = $self->{form};
     if ( $number > 1 ) {
         return $form->find_input( $name, undef, $number )->value();
-    } else {
+    }
+    else {
         return $form->value( $name );
     }
 } # value
@@ -1373,12 +1395,15 @@ sub click_button {
     my $request;
     if ( $args{name} ) {
         $request = $form->click( $args{name}, $args{x}, $args{y} );
-    } elsif ( $args{number} ) {
+    }
+    elsif ( $args{number} ) {
         my $input = $form->find_input( undef, 'submit', $args{number} );
         $request = $input->click( $form, $args{x}, $args{y} );
-    } elsif ( $args{input} ) {
+    }
+    elsif ( $args{input} ) {
         $request = $args{input}->click( $form, $args{x}, $args{y} );
-    } elsif ( $args{value} ) {
+    }
+    elsif ( $args{value} ) {
         my $i = 1;
         while ( my $input = $form->find_input(undef, 'submit', $i) ) {
             if ( $args{value} && ($args{value} eq $input->value) ) {
@@ -1459,14 +1484,14 @@ sub submit_form {
     }
 
     if ( my $form_number = $args{'form_number'} ) {
-        $self->form_number( $form_number ) ;
+        $self->form_number( $form_number ) or die;
     }
     elsif ( my $form_name = $args{'form_name'} ) {
-        $self->form_name( $form_name ) ;
+        $self->form_name( $form_name ) or die;
     }
 
     if ( my $fields = $args{'fields'} ) {
-        if ( ref $fields eq 'HASH' ) {
+        if ( isa( $fields, 'HASH' ) ) {
             $self->set_fields( %{$fields} ) ;
         } # TODO: What if it's not a hash?  We just ignore it silently?
     }
@@ -1474,7 +1499,8 @@ sub submit_form {
     my $response;
     if ( $args{button} ) {
         $response = $self->click( $args{button}, $args{x} || 0, $args{y} || 0 );
-    } else {
+    }
+    else {
         $response = $self->submit();
     }
 
@@ -1700,99 +1726,9 @@ sub update_html {
     $self->_extract_links();
     $self->_extract_images();
 
-    $self->_parse_html(); #For compatibility with folks that used to overload that method.
-
     return;
 }
 
-=head2 $mech->_update_page($request, $response)
-
-Updates all internal variables in $mech as if $request was just
-performed, and returns $response. The page stack is B<not> altered by
-this method, it is up to caller (e.g. L</request>) to do that.
-
-=cut
-
-sub _update_page {
-    my ($self, $request, $res) = @_;
-
-    $self->{req} = $request;
-    $self->{redirected_uri} = $request->uri->as_string;
-
-    $self->{res} = $res;
-
-    # These internal hash elements should be dropped in favor of
-    # the accessors soon. -- 1/19/03
-    $self->{status}  = $res->code;
-    $self->{base}    = $res->base;
-    $self->{ct}      = $res->content_type || "";
-
-    if ( $res->is_success ) {
-        $self->{uri} = $self->{redirected_uri};
-        $self->{last_uri} = $self->{uri};
-    } else {
-        if ( $self->{autocheck} ) {
-            $self->die( "Error ", $request->method, "ing ", $request->uri, ": ", $res->message );
-        }
-    }
-
-    $self->_reset_page;
-    if ($self->is_html) {
-        $self->update_html($res->content);
-    } else {
-        $self->{content} = $res->content;
-    }
-
-    return $res;
-} # _update_page
-
-
-=head2 $mech->_modify_request( $req )
-
-Modifies the request according to all the internal header mangling.
-
-=cut
-
-sub _modify_request {
-    my $self = shift;
-    my $req = shift;
-
-    # add correct Accept-Encoding header to restore compliance with
-    # http://www.freesoft.org/CIE/RFC/2068/158.htm
-    unless ( $req->header( 'Accept-Encoding' ) ) {
-        # Only allow "identity" for the time being
-        $req->header( 'Accept-Encoding', 'identity' );
-    }
-
-    my $last = $self->{last_uri};
-    if ( $last ) {
-        $last = $last->as_string if ref($last);
-        $req->header( Referer => $last );
-    }
-    while ( my($key,$value) = each %{$self->{headers}} ) {
-        if ( defined $value ) {
-            $req->header( $key => $value );
-        } else {
-            $req->remove_header( $key );
-        }
-    }
-
-    return $req;
-}
-
-
-=head2 $mech->_make_request()
-
-Convenience method to make it easier for subclasses like
-L<WWW::Mechanize::Cached> to intercept the request.
-
-=cut
-
-sub _make_request {
-    my $self = shift;
-    $self->SUPER::request(@_);
-}
-
 =head1 DEPRECATED METHODS
 
 This methods have been replaced by more flexible and precise methods.
@@ -1819,11 +1755,13 @@ sub follow {
     if ( $link =~ /^\d+$/ ) { # is a number?
         if ($link <= $#links) {
             $thislink = $links[$link];
-        } else {
+        }
+        else {
             $self->warn( "Link number $link is greater than maximum link $#links on this page ($self->{uri})" );
             return;
         }
-    } else {                        # user provided a regexp
+    }
+    else {                        # user provided a regexp
         LINK: foreach my $l (@links) {
             if ( defined($l->[1]) && $l->[1] =~ /$link/) {
                 $thislink = $l;     # grab first match
@@ -1860,22 +1798,102 @@ sub form {
     return $arg =~ /^\d+$/ ? $self->form_number($arg) : $self->form_name($arg);
 }
 
-=head2 $mech->_parse_html()
-
-An internal method that initializes forms and links given a HTML
-document.  Overriding this in your subclass is B<DEPRECATED>, better
-override L</update_html> instead in your new code.
-
-=cut
-
-sub _parse_html { }
-
-
 =head1 INTERNAL-ONLY METHODS
 
 These methods are only used internally.  You probably don't need to
 know about them.
 
+=head2 $mech->_update_page($request, $response)
+
+Updates all internal variables in $mech as if $request was just
+performed, and returns $response. The page stack is B<not> altered by
+this method, it is up to caller (e.g. L</request>) to do that.
+
+=cut
+
+sub _update_page {
+    my ($self, $request, $res) = @_;
+
+    $self->{req} = $request;
+    $self->{redirected_uri} = $request->uri->as_string;
+
+    $self->{res} = $res;
+
+    # These internal hash elements should be dropped in favor of
+    # the accessors soon. -- 1/19/03
+    $self->{status}  = $res->code;
+    $self->{base}    = $res->base;
+    $self->{ct}      = $res->content_type || "";
+
+    if ( $res->is_success ) {
+        $self->{uri} = $self->{redirected_uri};
+        $self->{last_uri} = $self->{uri};
+    }
+    else {
+        if ( $self->{autocheck} ) {
+            $self->die( "Error ", $request->method, "ing ", $request->uri, ": ", $res->message );
+        }
+    }
+
+    $self->_reset_page;
+    if ($self->is_html) {
+        $self->update_html($res->content);
+    }
+    else {
+        $self->{content} = $res->content;
+    }
+
+    return $res;
+} # _update_page
+
+
+=head2 $mech->_modify_request( $req )
+
+Modifies the request according to all the internal header mangling.
+
+=cut
+
+sub _modify_request {
+    my $self = shift;
+    my $req = shift;
+
+    # add correct Accept-Encoding header to restore compliance with
+    # http://www.freesoft.org/CIE/RFC/2068/158.htm
+    unless ( $req->header( 'Accept-Encoding' ) ) {
+        # Only allow "identity" for the time being
+        $req->header( 'Accept-Encoding', 'identity' );
+    }
+
+    my $last = $self->{last_uri};
+    if ( $last ) {
+        $last = $last->as_string if ref($last);
+        $req->header( Referer => $last );
+    }
+    while ( my($key,$value) = each %{$self->{headers}} ) {
+        if ( defined $value ) {
+            $req->header( $key => $value );
+        }
+        else {
+            $req->remove_header( $key );
+        }
+    }
+
+    return $req;
+}
+
+
+=head2 $mech->_make_request()
+
+Convenience method to make it easier for subclasses like
+L<WWW::Mechanize::Cached> to intercept the request.
+
+=cut
+
+sub _make_request {
+    my $self = shift;
+    $self->SUPER::request(@_);
+}
+
 =head2 $mech->_reset_page()
 
 Resets the internal fields that track page parsed stuff.
@@ -1953,7 +1971,7 @@ sub _image_from_token {
 
     if ( $tag eq "input" ) {
         my $type = $attrs->{type} or return;
-        return unless $type eq "submit";
+        return unless $type eq "image";
     }
 
     require WWW::Mechanize::Image;
@@ -2003,7 +2021,8 @@ sub _link_from_token {
 
         if ( $content =~ /^\d+\s*;\s*url\s*=\s*(\S+)/i ) {
             $url = $1;
-        } else {
+        }
+        else {
             undef $url;
         }
     } # meta
@@ -2242,8 +2261,14 @@ Here are modules that use or subclass Mechanize.  Let me know of any others:
 
 Acts as a proxy for web interaction, and then generates WWW::Mechanize scripts.
 
+=item * L<Win32::IE::Mechanize>
+
+Just like Mech, but using Microsoft Internet Explorer to do the work.
+
 =item * L<WWW::Bugzilla>
 
+=item * L<WWW::CheckSite>
+
 =item * L<WWW::Google::Groups>
 
 =item * L<WWW::Hotmail>
@@ -2307,6 +2332,9 @@ Scott Lanning,
 Rob Casey,
 Leland Johnson,
 Joshua Gatcomb,
+Julien Beasley,
+Abe Timmerman,
+Peter Stevens,
 and the late great Iain Truskett.
 
 =head1 COPYRIGHT
